"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connection = void 0;
const logger_1 = __importStar(require("./logger"));
const constants_1 = require("./constants");
class WSPingReceiver {
    constructor(failedProbesThreshold, interval, connection, logger) {
        this.failedProbesThreshold = failedProbesThreshold;
        this.pingsMissing = 0;
        this.interval = interval;
        this.connection = connection;
        this.logger = (logger) ? logger : new logger_1.default();
        if (!logger) {
            this.logger.setPrefix(() => "[WSPingReceiver]");
            this.logger.setVerbosity(logger_1.Verbosity.INFO);
        }
    }
    start() {
        if (this.failedProbesThreshold > 0 && this.interval > 0) {
            const receiver = this;
            this.checkInterval = window.setInterval(function () {
                receiver.checkPingsReceived();
            }, this.interval);
        }
    }
    stop() {
        if (this.checkInterval) {
            window.clearInterval(this.checkInterval);
            this.checkInterval = null;
        }
    }
    success() {
        this.pingsMissing = 0;
    }
    checkPingsReceived() {
        this.pingsMissing++;
        if (this.pingsMissing >= this.failedProbesThreshold) {
            this.failure();
        }
    }
    failure() {
        return __awaiter(this, void 0, void 0, function* () {
            let info;
            info = "Missing " + this.pingsMissing + " pings from server, connection seems to be down";
            this.logger.warn(info);
            yield this.connection.close({
                type: constants_1.SfuEvent.CONNECTION_FAILED,
                info: info
            });
        });
    }
}
class Connection {
    constructor(onMessage, onBinaryData, onError, onClose, logger) {
        this.connected = false;
        this.onMessage = onMessage;
        this.onBinaryData = onBinaryData;
        this.onError = onError;
        this.onClose = onClose;
        this.logger = (logger) ? logger : new logger_1.default();
        if (!logger) {
            this.logger.setPrefix(() => "[Connection]");
            this.logger.setVerbosity(logger_1.Verbosity.INFO);
        }
        this.pingChecker = null;
    }
    connect(options) {
        const timeout = options.timeout !== undefined ? options.timeout : constants_1.WS_CONNECTION_TIMEOUT;
        const failedProbesThreshold = options.failedProbesThreshold !== undefined ? options.failedProbesThreshold : constants_1.WS_PINGS_MISSING_THRESHOLD;
        const pingInterval = options.pingInterval !== undefined ? options.pingInterval : constants_1.WS_PING_INTERVAL_MS;
        this.ws = new WebSocket(options.url);
        this.ws.binaryType = 'arraybuffer';
        const that = this;
        return new Promise(function (resolve, reject) {
            if (that.ws.readyState === 3) {
                reject("Can't connect to websocket URL " + options.url);
            }
            if (timeout > 0) {
                that.connectionTimeout = window.setInterval(function () {
                    if (that.ws.readyState === 0) {
                        that.logger.warn("WS connection timeout");
                        that.ws.close();
                    }
                }, timeout);
            }
            that.ws.onerror = function (e) {
                if (that.connected) {
                    that.onError(e);
                }
                else {
                    reject(e);
                }
            };
            that.ws.onclose = function (e) {
                that.logger.debug("WebSocket connection closed, reason " + e.reason);
                if (that.connected) {
                    that.onClose(e);
                }
                else {
                    reject(e);
                }
            };
            that.ws.onopen = function () {
                clearInterval(that.connectionTimeout);
                let cConfig = {
                    appKey: options.appName,
                    mediaProviders: ["WebRTC"],
                    keepAlive: true,
                    authToken: null,
                    clientVersion: "0.5.28",
                    clientOSVersion: window.navigator.appVersion,
                    clientBrowserVersion: window.navigator.userAgent,
                    custom: options.custom
                };
                // Enable intervalic ping checking if ping options are set
                if (failedProbesThreshold > 0 && pingInterval > 0) {
                    that.pingChecker = new WSPingReceiver(failedProbesThreshold, pingInterval, that, that.logger);
                    that.pingChecker.start();
                }
                //connect to REST App
                that.send("connection", cConfig);
            };
            that.ws.onmessage = function (event) {
                let name;
                let msg;
                if (event.data instanceof ArrayBuffer) {
                    name = constants_1.InternalApi.BINARY_DATA;
                }
                else {
                    let obj = JSON.parse(event.data);
                    name = obj.message;
                    msg = obj.data;
                }
                switch (name) {
                    case "ping":
                        that.send("pong", null);
                        if (that.pingChecker) {
                            that.pingChecker.success();
                        }
                        break;
                    case "getUserData":
                        that.connected = true;
                        resolve(msg[0]);
                        break;
                    case "binaryData":
                        that.onBinaryData(name, event.data);
                        break;
                    default:
                        that.onMessage(name, msg);
                }
            };
        });
    }
    send(message, data) {
        this.logger.info("==>", message, data);
        if (this.ws) {
            this.ws.send(JSON.stringify({
                message: message,
                data: [data]
            }));
        }
    }
    ;
    sendBinaryData(data) {
        this.logger.info("==>", data.size);
        if (this.ws) {
            this.ws.send(data);
        }
    }
    close(error) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.ws) {
                // Stop pings checking if enabled
                if (this.pingChecker) {
                    this.pingChecker.stop();
                }
                // Close websocket connection
                const self = this;
                return new Promise(function (resolve) {
                    self.ws.onclose = function (e) {
                        self.logger.debug("WebSocket connection closed, reason " + e.reason);
                        if (self.connected) {
                            if (error) {
                                self.onError(error);
                            }
                            else {
                                self.onClose(e);
                            }
                        }
                        resolve();
                    };
                    self.ws.close();
                });
            }
        });
    }
}
exports.Connection = Connection;
//# sourceMappingURL=connection.js.map